/*
 * Created on: Jul 27, 2024
 */

#include "led.h"


#define FL3729_CS_OUTPUTS   15

#define LED_INIT_CURRENT    0
#define LED_INIT_SCALING    0xff



static const uint8_t rgb_map[9] = {0x04, 0x14, 0x01, 0x11, 0x07, 0x0a, 0x1a, 0x0d, 0x1d};
static const uint8_t cursor_map[3] = {0x16, 0x17, 0x18};

static const uint8_t LED_CIE_PWM_256I_256O[] = {
    0,    0,    0,    0,    0,    1,    1,    1,    1,    1,    1,    1,    1,    1,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    3,    3,    3,    3,    3,    3,    3,    3,    4,
    4,    4,    4,    4,    4,    5,    5,    5,    5,    5,    6,    6,    6,    6,    6,    7,
    7,    7,    7,    8,    8,    8,    8,    9,    9,    9,   10,   10,   10,   10,   11,   11,
   11,   12,   12,   12,   13,   13,   13,   14,   14,   15,   15,   15,   16,   16,   17,   17,
   17,   18,   18,   19,   19,   20,   20,   21,   21,   22,   22,   23,   23,   24,   24,   25,
   25,   26,   26,   27,   28,   28,   29,   29,   30,   31,   31,   32,   32,   33,   34,   34,
   35,   36,   37,   37,   38,   39,   39,   40,   41,   42,   43,   43,   44,   45,   46,   47,
   47,   48,   49,   50,   51,   52,   53,   54,   54,   55,   56,   57,   58,   59,   60,   61,
   62,   63,   64,   65,   66,   67,   68,   70,   71,   72,   73,   74,   75,   76,   77,   79,
   80,   81,   82,   83,   85,   86,   87,   88,   90,   91,   92,   94,   95,   96,   98,   99,
  100,  102,  103,  105,  106,  108,  109,  110,  112,  113,  115,  116,  118,  120,  121,  123,
  124,  126,  128,  129,  131,  132,  134,  136,  138,  139,  141,  143,  145,  146,  148,  150,
  152,  154,  155,  157,  159,  161,  163,  165,  167,  169,  171,  173,  175,  177,  179,  181,
  183,  185,  187,  189,  191,  193,  196,  198,  200,  202,  204,  207,  209,  211,  214,  216,
  218,  220,  223,  225,  228,  230,  232,  235,  237,  240,  242,  245,  247,  250,  252,  255,
};



uint16_t rgb[9][3] = {0};
uint8_t  cursor[3] = {0};

static uint8_t led_updated = 0;


void led_init()
{
    uint8_t i;
    uint8_t buf[FL3729_CS_OUTPUTS] = {0xff};

    is31fl3729_init(FL3729_ADDR,
                    FL3729_CONF_SSD_NRML | FL3729_CONF_OSDE_OFF | FL3729_CONF_MATRIX_2x16,
                    LED_INIT_CURRENT);

    for (i = 0; i < FL3729_CS_OUTPUTS; i++) buf[i] = LED_INIT_SCALING;
    is31fl3729_set_scaling_current_multi(FL3729_ADDR, buf, FL3729_CS_OUTPUTS);

}

void led_is_updated()
{
    led_updated = 1;
}

void led_send()
{
    uint8_t i;
    uint8_t o;
    uint8_t out[16*2];

    // only render when there's something to render
    if (led_updated) {
        // stuff RGB outputs
        for (i = 0; i < 9; i++) {
            o = (rgb_map[i] - 1) & 0x1f;
            out[o + 0] = rgb[i][0];
            out[o + 1] = rgb[i][1];
            out[o + 2] = rgb[i][2];
        }

        // stuff cursor outputs
        out[cursor_map[0]] = cursor[0];
        out[cursor_map[1]] = cursor[1];
        out[cursor_map[2]] = cursor[2];

        is31fl3729_set_outputs(FL3729_ADDR, 1, out, 16+15);

        led_updated = 0;
    }
}

uint8_t led_gamma(uint8_t in)
{
    return LED_CIE_PWM_256I_256O[in];
}
